@@ -19,6 +19,7 @@ except ImportError:
 
 import pytest
 from nucos import unit_conversion
+from nucos.unit_data import ConvertDataUnits
 
 RELTOL = 1e-5
 ABSTOL = 1e-5  # not used, but maybe some day?
@@ -148,8 +149,10 @@ KnownValues = [
     ("Dynamic Viscosity", "mPa s", "Pa s", 1.0, 0.001),
 
     ("temperature", "F", "C", 32, 0),
+    ("temperature", "°F", "°C", 32, 0),
     ("temperature", "F", "C", 212, 100),
     ("temperature", "C", "K", 0, 273.15),
+    ("temperature", "°C", "K", 0, 273.15),
     ("temperature", "F", "K", 32, 273.15),
 
     ("deltatemperature", "F", "C", 1.0, (5.0 / 9.0)),
@@ -286,8 +289,14 @@ def test_new_api_oneshot():
     assert isclose(unit_conversion.convert('meter second-1', 'knot', 1),
                    1.94384, rel_tol=RELTOL)
 
-    assert isclose(unit_conversion.convert('m/s', 'knot', 1), 1.94384, rel_tol=RELTOL)
+    assert isclose(unit_conversion.convert('m/s', 'knot', 1),
+                   1.94384, rel_tol=RELTOL)
 
+    assert isclose(unit_conversion.convert('g/kg', 'ppm', 1),
+                   1000, rel_tol=RELTOL)
+
+    assert isclose(unit_conversion.convert('ml/l', '%', 1),
+                   0.1, rel_tol=RELTOL)
 
 @pytest.mark.parametrize('unit_type, unit1, unit2, value, new_value',
                          KnownValues)
@@ -303,7 +312,6 @@ def test_new_api(unit_type, unit1, unit2, value, new_value):
                                                'volumefraction',
                                                'deltatemperature'):
         return
-    # now do the test:
     assert isclose(unit_conversion.convert(unit1, unit2, value),
                    new_value, rel_tol=RELTOL)
 
@@ -407,7 +415,7 @@ class TestNewConvertAPI():
                               (0, 0, 0),
                               ])
     def test_NotSupportedUnitError(self, args):
-        with pytest.raises(unit_conversion.NotSupportedUnitError):
+        with pytest.raises(unit_conversion.UnitConversionError):
             unit_conversion.convert(*args)
 
     @pytest.mark.parametrize('args',
@@ -430,7 +438,7 @@ def test_GetUnitTypes():
     assert "Volume" in types
 
 
-def test_GetUnitNames():
+def test_GetUnitNames_length():
     # note: not testing all of them
     #       either all types or all names
     names = unit_conversion.GetUnitNames('Length')
@@ -440,6 +448,7 @@ def test_GetUnitNames():
     assert "foot" in names
     assert "feet" not in names
 
+def test_GetUnitNames_area():
     names = unit_conversion.GetUnitNames('Area')
     # print(names)
 
@@ -449,7 +458,29 @@ def test_GetUnitNames():
     assert "feet" not in names
 
 
+def test_GetUnitNames_kin_vis():
+    names = unit_conversion.GetUnitNames('Kinematic Viscosity')
+    print(names)
+
+    assert 'Stoke' in names
+    assert 'square centimeter per second' in names
+
+
+def test_proper_unit_type_names():
+    """
+    all unit types should be title cased
+
+    This is assumed elsewhere in the code
+    """
+    for key in ConvertDataUnits.keys():
+        assert key == key.title(), f"{key} is not title cased"
+
+
 def test_GetUnitAbbreviation():
+    """
+    NOTE: GetUnitAbbreviation is a deprecated name
+          get_abbreviation is the correct one now.
+    """
     names = [('Length', 'meter', 'm'),
              ('Volume', 'cubic meter', 'm³'),
              ('Time', 'second', 's'),
@@ -459,7 +490,7 @@ def test_GetUnitAbbreviation():
 
     for unit_type, unit, abrv in names:
         print(unit_type, unit, abrv)
-        assert abrv == unit_conversion.GetUnitAbbreviation(unit_type, unit)
+        assert abrv == unit_conversion.get_abbreviation(unit, unit_type)
 
 
 def test_FindUnitTypes():
@@ -547,3 +578,150 @@ def test_InvalidUnitTypeError():
     err = str(unit_conversion.InvalidUnitTypeError('feeet'))
 
     assert err == 'The unit type: feeet is not in the UnitConversion database'
+
+
+class Test_FindAllUnitNames():
+
+    UNIT_NAMES = unit_conversion.FindAllUnitNames()
+
+    def test_length(self):
+        names = self.UNIT_NAMES['length']
+
+        assert "meter" in names
+        assert "nauticalmiles" in names
+        assert "kilometer" in names # primary name
+
+        assert "metter" not in names
+
+    def test_dynamic_viscosity(self):
+        names = self.UNIT_NAMES['dynamicviscosity']
+
+        print(names)
+        "Pascal second" in names # primary name
+        assert 'kg/(m s)' in names
+        assert 'cP' in names
+
+
+def test_get_supported_names():
+    """
+    not the least bit comprehensive, but at shows dict is built
+    """
+    names = unit_conversion.get_supported_names('Volume')
+    names2 = unit_conversion.get_supported_names('volume')
+
+    assert names == names2
+    assert 'oz' in names
+    assert 'meter' not in names
+    assert "cubic meter" in names
+
+    names = unit_conversion.get_supported_names('Concentration')
+    assert 'parts per thousand' in names
+    assert 'part per thousand' in names
+
+
+@pytest.mark.parametrize(("unit_type", "unit"),[('Length', 'meter'),
+                                                ('length', 'cM'),
+                                                ('oil Concentration', 'micron'),
+                                                ])
+def test_is_supported_unit(unit_type, unit):
+    assert unit_conversion.is_supported_unit(unit_type, unit)
+
+
+@pytest.mark.parametrize(("unit_type", "unit"), [('Length', 'metter'),
+                                                 ('length', 'mcM'),
+                                                 ('oil Concentration', 'sheen'),
+                                                 ])
+def test_is_supported_unit_not(unit_type, unit):
+    assert not unit_conversion.is_supported_unit(unit_type, unit)
+
+
+@pytest.mark.parametrize(("unit", "primary_name"), [('kg', 'kilogram'),
+                                                    ('us ton', 'ton (US)'),
+                                                    ('oz', 'ounce'),
+                                                    ('fluid oz', 'fluid ounce'),
+                                                    ("ukgal", "gallon (UK)"),
+                                                    ])
+def test_get_primary_name_there(unit, primary_name):
+    pn = unit_conversion.get_primary_name(unit)
+    assert pn == primary_name
+
+
+@pytest.mark.parametrize(("unit"), [('kgss',),
+                                    ('us tonage',),
+                                    ])
+def test_get_primary_name_not_there(unit):
+    with pytest.raises(unit_conversion.NotSupportedUnitError):
+        pn = unit_conversion.get_primary_name(unit)
+        assert pn == 'nothing'
+
+
+@pytest.mark.parametrize(("unit", "unit_type", "primary_name"),
+                         [('kg', 'mass', 'kilogram'),
+                          ('oz', 'mass', 'ounce'),
+                          ('oz', 'volume', 'fluid ounce'),
+                          ])
+def test_get_primary_name_specify_unit_type(unit, unit_type, primary_name):
+    pn = unit_conversion.get_primary_name(unit, unit_type=unit_type)
+    assert pn == primary_name
+
+
+@pytest.mark.parametrize(('unit', 'abbv'),
+                         [('kilogram', 'kg'),
+                          ('barrel (petroleum)', "bbl"),
+                          ('milligram', 'mg'),
+                          ('microgram', 'µg'),
+                          ('ton (UK)', "ukton"),
+                          ('bar', 'bar'),  # this one has no synonyms
+                          ])
+def test_get_abbreviation(unit, abbv):
+    assert unit_conversion.get_abbreviation(unit) == abbv
+
+def test_abbreviation_for_webGNOME():
+    """
+    Just to make sure we don't change anyting that might break WebGNOME
+    """
+    units = ['bbl', 'gal', 'm³', 'kg', 'ton', 'mt']
+
+    for u in units:
+        assert u == unit_conversion.get_abbreviation(u)
+
+
+def test_get_primary_names():
+    result = unit_conversion.get_primary_names('length')
+    assert set(result) == set([
+        'meter',
+        'centimeter',
+        'millimeter',
+        'micron',
+        'kilometer',
+        'foot',
+        'inch',
+        'yard',
+        'mile',
+        'nautical mile',
+        'fathom',
+        'latitude degree',
+        'latitude minute',
+    ])
+
+def test_get_primary_names_with_space():
+    result = unit_conversion.get_primary_names("Mass Discharge")
+    assert set(result) == {'gram per second', 'kilogram per second'}
+
+
+def test_get_unit_type():
+    """
+    not much, but something
+    """
+    unit_type = unit_conversion.get_unit_type('m/s')
+
+    assert unit_type == "velocity"
+
+def test_get_unit_type_bad():
+    """
+    not much, but something
+    """
+    with pytest.raises(unit_conversion.NotSupportedUnitError):
+        unit_type = unit_conversion.get_unit_type('fred')
+
+
