@@ -47,36 +47,8 @@ from .lat_long import (LatLongConverter,
                        format_lon_dms,
                        )
 
-__all__ = ['DensityConverterClass',
-           'InvalidUnitTypeError',
-           'Latitude',
-           'ConverterClass',
-           'is_supported',
-           'Converters',
-           'GetUnitNames',
-           'NotSupportedUnitError',
-           'UNIT_TYPES',
-           'TempConverterClass',
-           'Simplify',
-           'GetUnitAbbreviation',
-           'Convert',
-           'FindUnitTypes',
-           'lat_long',
-           'LatLongConverter',
-           'Longitude',
-           'ConvertDataUnits',
-           'InvalidUnitError',
-           'is_same_unit',
-           'GetUnitTypes',
-           'OilQuantityConverter',
-           'convert',
-           'UnitConversionError',
-           'format_lat',
-           'format_lon',
-           'format_lat_dms',
-           'format_lon_dms',
-           ]
-
+# reset this to get some more print statements on import
+DEBUG = False
 
 # A few utilities
 def Simplify(string):
@@ -85,6 +57,10 @@ def Simplify(string):
 
     returns the string with the whitespace and capitalization removed
     """
+    # This should perhaps be made smarter -- but what to do with whitespace?
+    # "m / s" -- "m/s"
+    # "meters-1" "meter s-1"
+
     try:
         return "".join(string.lower().split()).replace(".", "")
     except AttributeError:
@@ -100,6 +76,31 @@ def GetUnitTypes():
     return list(ConvertDataUnits.keys())
 
 
+def get_unit_type(unit):
+    """
+    Return the unit type for a given unit name.
+
+    Note that this will only work for unambiguous units
+
+    :param unit: the unit you want the type of
+    :type unit: str
+
+    In [2]: nucos.get_unit_type('meter')
+    Out[2]: 'length'
+    """
+    unit = Simplify(unit)
+
+    try:
+        unit_type = UNIT_TYPES[unit]
+    except KeyError:
+        raise NotSupportedUnitError(unit)
+    return unit_type
+
+
+def get_unit_types():
+    return list(PRETTY_UNIT_TYPES.values())
+
+
 def GetUnitNames(UnitType):
     """
     returns a list of all the units available for a given unit type available
@@ -108,19 +109,28 @@ def GetUnitNames(UnitType):
 
     a unit of mass would be "kilogram", "slug", etc.
     """
-    UnitType.capitalize()
-    return list(ConvertDataUnits[UnitType].keys())
+    return list(ConvertDataUnits[UnitType.title()].keys())
+
 
 
 def FindUnitTypes():
     """
-    returns a mapping of all the unit names to the unit types
+    Returns a mapping of all the unit names to the unit types
 
-    raises an exception if there is more than one option -- this will check
+    Raises an exception if there is more than one option -- this will check
     the unit database for duplicated names
 
+    Primarily used to do conversions without specifying the unit types
+
     Usually not called from user code.
     """
+
+    # compute units that are unique to Mass and Volume Fraction
+    mf = ConvertDataUnits["Mass Fraction"]
+    vf = ConvertDataUnits["Volume Fraction"]
+    # not_to_skip = {Simplify(u) for u in mf.keys() ^ vf.keys()}
+    to_skip = {Simplify(u) for u in mf.keys() & vf.keys()}
+
     unit_types = {}
 
     for unit_type, unit_data in ConvertDataUnits.items():
@@ -134,8 +144,8 @@ def FindUnitTypes():
         #   that conflict with Concentration & Concentration In Water.
         if unit_type in ('oilconcentration',
                          'concentrationinwater',
-                         'massfraction',
-                         'volumefraction',
+                         #'massfraction',
+                         #'volumefraction',
                          'deltatemperature',
                          'dimensionless',
                          ):
@@ -145,9 +155,14 @@ def FindUnitTypes():
             # strip out whitespace and capitalization
             pname = Simplify(pname)
 
-            # add the primary name:
-            unit_types[pname] = unit_type
+            if (unit_type in {'massfraction', 'volumefraction'}
+                    and pname in to_skip):
+                continue
 
+            # add the primary name:
+            if pname in unit_types:
+                raise ValueError(f"Duplicate primary name in units table: {pname}")
+            unit_types[pname] = unit_type
             # now the synonyms:
             for n in data[1]:
                 n = Simplify(n)
@@ -156,7 +171,11 @@ def FindUnitTypes():
                 if (unit_type, n) in [("volume", "oz"),
                                       ("density", "s")]:
                     continue
-
+                if DEBUG:
+                    try:
+                        print(f"Adding: {unit_type}: {n}")
+                    except UnicodeEncodeError:
+                        print(f"Adding: {unit_type}: {n}".encode("utf-8"))
                 if n in unit_types:
                     raise ValueError("Duplicate name in units table: %s" % n)
 
@@ -165,7 +184,101 @@ def FindUnitTypes():
     return unit_types
 
 
+def FindAllUnitNames():
+    """
+    returns a mapping of all the valid unit names to unit type
+
+    Usually not called from user code.
+    """
+    unit_names = {}
+
+    for unit_type, unit_data in ConvertDataUnits.items():
+        unit_type = Simplify(unit_type)
+
+        for pname, data in unit_data.items():
+            # add the primary name:
+            if DEBUG:
+                print("adding:", pname)
+            unit_names.setdefault(unit_type, []).append(pname)
+            # now the synonyms:
+            unit_names[unit_type].extend(data[1])
+    return unit_names
+
+
 UNIT_TYPES = FindUnitTypes()
+UNIT_NAMES = FindAllUnitNames()
+PRETTY_UNIT_TYPES = {Simplify(unit_type): unit_type for unit_type in ConvertDataUnits.keys()}
+
+
+def is_supported_unit(unit_type, unit):
+    """
+    checks if a unit name is supported for the given unit type
+    """
+    unit_type = Simplify(unit_type)
+    converter = Converters[unit_type]
+    return Simplify(unit) in converter.Synonyms
+
+
+def get_supported_names(unit_type):
+    """
+    Returns the list of all supported unit names for a unit_type
+    """
+    return UNIT_NAMES[Simplify(unit_type)]
+
+def get_primary_name(unit, unit_type=None):
+    """
+    Returns the primary name for a given unit.
+
+    This is usually the spelled out version, e.g.
+    kilogram
+    """
+    unit = Simplify(unit)
+    if unit_type is None:
+        unit_type = UNIT_TYPES[unit]
+    else:
+        unit_type = Simplify(unit_type)
+    return Converters[unit_type].GetPrimaryName(unit)
+
+def get_primary_names(unit_type):
+    """
+    return the primary names of all the units supported for a given unit type
+    """
+    p_names = []
+
+    unit_type = Simplify(unit_type)
+    unit_type = PRETTY_UNIT_TYPES[unit_type]
+    unit_data = ConvertDataUnits[unit_type]
+    for pname, data in unit_data.items():
+        p_names.append(pname)
+
+    return p_names
+
+
+def get_abbreviation(unit, unit_type=None):
+    """
+    return the standard abbreviated form for a given unit
+
+    :param unit: name of the unit
+    :type unit: str
+
+    :param unit=None: Unit type -- helpful if the unit name conflicts,
+                      e.g. oz: weight or volume?
+    :type unit: str
+    """
+    unit = Simplify(unit)
+    if unit_type is None:
+        unit_type = UNIT_TYPES[unit]
+    else:
+        unit_type = Simplify(unit_type)
+
+    unit_type = PRETTY_UNIT_TYPES[unit_type]
+    unit = get_primary_name(unit, unit_type)
+    synonyms = ConvertDataUnits[unit_type][unit][1]
+    try:
+        return synonyms[0]
+    except IndexError:
+        # If there are no synonyms, return the primary name
+        return unit
 
 
 def GetUnitAbbreviation(unit_type, unit):
@@ -175,6 +288,7 @@ def GetUnitAbbreviation(unit_type, unit):
     :param unit_type: the type of unit: "mass", "length", etc.
     :param unit: the unit you want the abbreviation for: "gram", etc.
     """
+    warnings.warn('`GetUnitAbbreviation` is deprecated -- use `get_abbreviation`', DeprecationWarning)
     return ConvertDataUnits[unit_type][unit][1][0]
 
 
@@ -228,10 +342,12 @@ class ConverterClass:
 
         self.Synonyms = {}
         self.Convertdata = {}
+        self.PrettyNames = {}
 
         for PrimaryName, data in UnitsDict.items():
             # strip out whitespace and capitalization
             Pname = Simplify(PrimaryName)
+            self.PrettyNames[Pname] = PrimaryName
 
             self.Convertdata[Pname] = data[0]
             self.Synonyms[Pname] = Pname
@@ -270,6 +386,9 @@ class ConverterClass:
 
         return Value * self.Convertdata[FromUnit] / self.Convertdata[ToUnit]
 
+    def GetPrimaryName(self, unit):
+        return self.PrettyNames[self.Synonyms[Simplify(unit)]]
+
 
 # the special case classes:
 class TempConverterClass(ConverterClass):
@@ -288,6 +407,7 @@ class TempConverterClass(ConverterClass):
         :param ToUnit: the unit you want the value converted to
         :param Value: the original value
         """
+#        breakpoint()
         FromUnit = Simplify(FromUnit)
         ToUnit = Simplify(ToUnit)
 
@@ -465,14 +585,14 @@ def convert(unit1, unit2, value, unit_type=None):
         except KeyError:
             raise NotSupportedUnitError(unit1)
 
-        try:
-            unit_type2 = UNIT_TYPES[unit2]
-        except KeyError:
-            raise NotSupportedUnitError(unit2)
+        # try:
+        #     unit_type2 = UNIT_TYPES[unit2]
+        # except KeyError:
+        #     raise NotSupportedUnitError(unit2)
 
-        if unit_type != unit_type2:
-            raise UnitConversionError("Cannot convert {0} to {1}"
-                                      .format(unit1, unit2))
+        # if unit_type != unit_type2:
+        #     raise UnitConversionError("Cannot convert {0} to {1}"
+        #                               .format(unit1, unit2))
 
         unit_type = Simplify(unit_type)
     else:
@@ -499,7 +619,7 @@ def Convert(*args, **kwargs):
 
     for new code, use convert()
     """
-    warnings.warn('"Convert" is deprecated -- use "convert()"',
+    warnings.warn('"Convert()" is deprecated -- use "convert()"',
                   DeprecationWarning)
 
     return convert(*args, **kwargs)
